#region using
using DataDynamics.ActiveReports;
using DataDynamics.ActiveReports.Document;
using Kercia.AlphaVote.AlphaVoteMetierUtils;
using Kercia.AlphaVote.Infrastructure;
using Kercia.AlphaVote.Metier;
using Kercia.AlphaVote.Metier.Entites;
using Kercia.AlphaVote.Metier.Entites.Elections;
using Kercia.AlphaVote.Ressources;
using Kercia.AlphaVote.Resultats;
using Kercia.AlphaVote.Utils;
using NHibernate.Criterion;
using System;
using System.Collections.Generic;
using UtilsActiveReport;

#endregion

namespace Kercia.AlphaVote.Documents.Cerfa
{
    /// <summary>
    /// Summary description for ArCerfaCSE.
    /// </summary>
    public partial class ArCerfaCSE : Report
    {
        private ResultatElectionCSE _ResultatElectionListe;
        private bool premierePage = true;
        private ElectionCSE _Election;
        private XmlResultats _XmlResultatsTour1;

        public ArCerfaCSE(ResultatElectionCSE resultat)
        {
            InitializeComponent();
            Document.Printer.PrinterName = "";
            Document.CacheToDisk = false;
            _ResultatElectionListe = resultat;
            _Election = AccesMetier.SessionVote.GetCriteriaVotes(typeof(ElectionCSE),id:resultat.Vote.Id).UniqueResult<ElectionCSE>();
            // Entte du document selon le type de vote
            AffecterImageEnteteDocument();
            RemplirZonesEtablissementCollege();
            picPageVierge2ndTour.Image = ImageManip.SafeLoadImageFromFile(RessourcesMngrImages.GetPathCerfaCSEPage2ndTour(_Election.TypeVote.TypeVoteInterne.CodeType.ToString(), _Election.Titulaires.Value));

            // Remplissage de la zone des donnes globales des rsultats
            tbNbInscrits.Text = tbNbInscrits2.Text = resultat.NbInscrits.ToString();
            tbNbVotants.Text = resultat.NbVotants.ToString();
            tbNbBlancsNuls.Text = resultat.NbBlancsOuNuls.ToString();
            tbNbValables.Text = tbNbValables2.Text = resultat.NbBulletinsValables.ToString();
            tbCarenceOui.Text = TextePourComportementCheckbox(resultat.Carence);
            tbCarenceNon.Text = TextePourComportementCheckbox(!resultat.Carence);
            decimal partieDecimaleQuorum = resultat.Quorum - (int)resultat.Quorum;
            tbQuorum.Text = partieDecimaleQuorum == 0 ? ((int)resultat.Quorum).ToString() : resultat.Quorum.ToString("F2");
            tbQuorumAtteintNon.Text = TextePourComportementCheckbox(!resultat.QuorumAtteint);
            tbQuorumAtteintOui.Text = TextePourComportementCheckbox(resultat.QuorumAtteint);
            tbNbListesPresentees.Text = resultat.NbListesPresentees.ToString();
            tbNbSiegesAPourvoir.Text = tbNbSiegesAPourvoir2.Text = resultat.NbSiegesAPourvoir.ToString();
            tbQuotientElectoral.Text = resultat.QuotientElectoral.ToString("F2");
            // Si 1er tour -> Gnration du fichier xml rsultats qui permettra de remplir la 1re page du cerfa du 2nd tour.
            XmlResultats xmlResultats = null;
            if (_Election.BureauVote.SessionVote.NumeroTour == 1)
            {
                xmlResultats = new XmlResultats(_Election.BureauVote.IdentifiantRessource, _Election.Id, _Election.BureauVote.SessionVote.NumeroTour);
                xmlResultats.DateScrutin = AccesMetier.SessionVote.CarenceTotale ? _Election.BureauVote.DateFinTour1.ToString("dd/MM/yyyy") : DateTime.Now.ToString("dd/MM/yyyy");
                xmlResultats.NbInscrits = tbNbInscrits.Text;
                xmlResultats.NbVotants = tbNbVotants.Text;
                xmlResultats.NbBlancsNuls = tbNbBlancsNuls.Text;
                xmlResultats.NbSuffragesValables = tbNbValables.Text;
                xmlResultats.Carence = resultat.Carence ? "Oui" : "Non";
                xmlResultats.Quorum = resultat.QuorumAtteint ? "Oui" : "Non";
                xmlResultats.QuorumValeur = tbQuorum.Text;
                xmlResultats.NbListes = tbNbListesPresentees.Text;
                xmlResultats.NbSieges = tbNbSiegesAPourvoir.Text;
                xmlResultats.QuotientElectoral = tbQuotientElectoral.Text;
            }
            if (_Election.BureauVote.SessionVote.NumeroTour == 1 && AccesMetier.SessionVote.CarenceTotale)
            {
                tbJourPremierTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateFinTour1.ToString("dd"), 2);
                tbMoisPremierTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateFinTour1.ToString("MM"), 2);
                tbAnneePremierTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateFinTour1.ToString("yyyy"), 2);
            }
            else
            {
                tbJourPremierTour.Text = EclaterPourImpressionCases(DateTime.Now.ToString("dd"), 2);
                tbMoisPremierTour.Text = EclaterPourImpressionCases(DateTime.Now.ToString("MM"), 2);
                tbAnneePremierTour.Text = EclaterPourImpressionCases(DateTime.Now.ToString("yyyy"), 2);
            }
            tbHeuresOuverture1erTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateDebutTour1.ToString("HH"), 1);
            tbMinutesOuverture1erTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateDebutTour1.ToString("mm"), 1);
            tbHeureCloture1erTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateFinTour1.ToString("HH"), 1);
            tbMinutesCloture1erTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateFinTour1.ToString("mm"), 1);
            // Impression d'informations supplmentaires
            try
            {
                if (resultat != null && resultat.Vote != null)
                {
                    RemplirInformationsSupplementaires(resultat.Vote.BureauVote);
                }
            }
            catch (Exception ex)
            {
                LoggerUniversel.Error("*** Gnration Cerfa *** Impossible d'imprimer les informations supplmentaires", ex);
            }
            //subrepResListesCandidats.Report = new ArCerfaResListes(resultat, xmlResultats);
        }

        public ArCerfaCSE(ResultatElectionCSE resultat2ndTour, XmlResultats xmlResultats1erTour)
        {
            InitializeComponent();
            Document.Printer.PrinterName = "";
            Document.CacheToDisk = false;
            _ResultatElectionListe = resultat2ndTour;
            // Entte du document selon le type de vote
            AffecterImageEnteteDocument();
            _Election = AccesMetier.SessionVote.GetCriteriaVotes(typeof(ElectionCSE), id:resultat2ndTour.Vote.Id).UniqueResult<ElectionCSE>();
            RemplirZonesEtablissementCollege();
            // Remplissage de la zone des donnes globales des rsultats
            if (xmlResultats1erTour != null)
            {
                _XmlResultatsTour1 = xmlResultats1erTour;
                tbNbInscrits.Text = tbNbInscrits2.Text = xmlResultats1erTour.NbInscrits;
                tbNbVotants.Text = xmlResultats1erTour.NbVotants;
                tbNbBlancsNuls.Text = xmlResultats1erTour.NbBlancsNuls;
                tbNbValables.Text = tbNbValables2.Text = xmlResultats1erTour.NbSuffragesValables;
                tbCarenceOui.Text = TextePourComportementCheckbox(xmlResultats1erTour.Carence == "Oui");
                tbCarenceNon.Text = TextePourComportementCheckbox(xmlResultats1erTour.Carence == "Non");
                tbQuorum.Text = xmlResultats1erTour.QuorumValeur;
                tbQuorumAtteintNon.Text = TextePourComportementCheckbox(xmlResultats1erTour.Quorum == "Non");
                tbQuorumAtteintOui.Text = TextePourComportementCheckbox(xmlResultats1erTour.Quorum == "Oui");
                tbNbListesPresentees.Text = xmlResultats1erTour.NbListes;
                tbNbSiegesAPourvoir.Text = tbNbSiegesAPourvoir2.Text = xmlResultats1erTour.NbSieges;
                tbQuotientElectoral.Text = xmlResultats1erTour.QuotientElectoral;
                DateTime dateScrutinTour1 = DateTime.Now;
                if (!string.IsNullOrEmpty(xmlResultats1erTour.DateScrutin) && DateTime.TryParse(xmlResultats1erTour.DateScrutin, out dateScrutinTour1)) 
                {
                    tbJourPremierTour.Text = EclaterPourImpressionCases(dateScrutinTour1.ToString("dd"), 2);
                    tbMoisPremierTour.Text = EclaterPourImpressionCases(dateScrutinTour1.ToString("MM"), 2);
                    tbAnneePremierTour.Text = EclaterPourImpressionCases(dateScrutinTour1.ToString("yyyy"), 2);
                }
                tbHeuresOuverture1erTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateDebutTour1.ToString("HH"), 1);
                tbMinutesOuverture1erTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateDebutTour1.ToString("mm"), 1);
                tbHeureCloture1erTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateFinTour1.ToString("HH"), 1);
                tbMinutesCloture1erTour.Text = EclaterPourImpressionCases(_Election.BureauVote.DateFinTour1.ToString("mm"), 1);
            }
            // Impression d'informations supplmentaires
            try
            {
                if (resultat2ndTour != null && resultat2ndTour.Vote != null)
                {
                    RemplirInformationsSupplementaires(resultat2ndTour.Vote.BureauVote.VersionTourPrecedent != null ? resultat2ndTour.Vote.BureauVote.VersionTourPrecedent : resultat2ndTour.Vote.BureauVote);
                }
            }
            catch (Exception ex)
            {
                LoggerUniversel.Error("*** Gnration Cerfa *** Impossible d'imprimer les informations supplmentaires", ex);
            }
            //subrepResListesCandidats.Report = new ArCerfaResListes(xmlResultats1erTour);
        }

        private void RemplirZonesEtablissementCollege() 
        {
            // Remplissage de la zone tablissement
            Etablissement etablissement = null;
            if (_Election.EtablissementCollege != null)
            {
                etablissement = _Election.EtablissementCollege.Etablissement;
            }
            if (etablissement != null)
            {
                tbRaisonSociale.Text = etablissement.CerfaRaisonSociale;
                // "Scurit" pour le cas o la raison sociale spcifique Cerfa ne serait pas renseigne
                if (string.IsNullOrEmpty(tbRaisonSociale.Text))
                {
                    tbRaisonSociale.Text = etablissement.RaisonSociale;
                }
                tbAdresseL1.Text = etablissement.AdresseL1 + "\n" + etablissement.AdresseL2;
                tbCodePostal.Text = EclaterPourImpressionCases(etablissement.CodePostal, 3);
                tbVille.Text = etablissement.Ville;
                tbSiretPart1.Text = EclaterPourImpressionCases(StringHelper.GetPartNumeroSiret(etablissement.NoSiret, 1), 2);
                tbSiretPart2.Text = EclaterPourImpressionCases(StringHelper.GetPartNumeroSiret(etablissement.NoSiret, 2), 2);
                tbSiretPart3.Text = EclaterPourImpressionCases(StringHelper.GetPartNumeroSiret(etablissement.NoSiret, 3), 2);
                tbSiretPart4.Text = EclaterPourImpressionCases(StringHelper.GetPartNumeroSiret(etablissement.NoSiret, 4), 2);
                tbIdcc.Text = EclaterPourImpressionCases(etablissement.Idcc, 2);
                if (etablissement.DatePrecedentScrutin.HasValue)
                {
                    tbJourPrecScrutin.Text = EclaterPourImpressionCases(etablissement.DatePrecedentScrutin.Value.ToString("dd"), 2);
                    tbMoisPrecScrutin.Text = EclaterPourImpressionCases(etablissement.DatePrecedentScrutin.Value.ToString("MM"), 2);
                    tbAnneePrecScrutin.Text = EclaterPourImpressionCases(etablissement.DatePrecedentScrutin.Value.ToString("yyyy"), 2);
                }
                tbDureeMandatElus.Text = etablissement.DureeMandatElus.ToString();
                // Le calcul du nombre de collges concerns par cet tablissement tient compte de l'instance considre (CSE)               
                 tbNbColleges.Text = etablissement.GetCriteriaCollegesLieeElection()
                                                     .SetProjection(Projections.CountDistinct("coll.Id"))
                                                     .UniqueResult<int>()
                                                     .ToString();
                
                RemplirSiretsAssocies(etablissement);
            }
            // Remplissage de la zone collge
            College college = null;
            if (_Election.EtablissementCollege != null)
            {
                college = _Election.EtablissementCollege.College;
            }
            if (college != null)
            {
                if (college.DenominationCollegeLegal.HasValue)
                {
                        switch (college.DenominationCollegeLegal)
                        {
                            case CollegeLegal.CollegeUnique:
                                tbCollegeUniqueCSE.Text = TextePourComportementCheckbox(true);
                                break;
                            case CollegeLegal.PremierCollege:
                                tbCollege1CSE.Text = TextePourComportementCheckbox(true);
                                break;
                            case CollegeLegal.SecondCollege:
                                tbCollege2CSE.Text = TextePourComportementCheckbox(true);
                                break;
                            // On affiche le troisime collge pour le type d'lection CSE
                            case CollegeLegal.TroisiemeCollege:
                                    tbCollege3CSE.Text = TextePourComportementCheckbox(true);                                
                                break;
                            case CollegeLegal.Autre:
                                    tbCollegeAutreCSE.Text = TextePourComportementCheckbox(true);
                                break;
                        }

                        // Si le type d'lection n'est pas DU alors on n'affiche pas la checkbox correspondant au troisime collge DU
                        if (NHibernateManip.TypeReel(_Election) != typeof(ElectionCSE))
                        {
                            tbCollege3CSE.Visible = false;
                        }

                        tbCollegeUniqueCSE.Visible = tbCollege1CSE.Visible = tbCollege2CSE.Visible =  true;
                    }
                
                tbCompoCol1.Text = TextePourComportementCheckbox(college.CompositionOuvriers);
                tbCompoCol2.Text = TextePourComportementCheckbox(college.CompositionEmployes);
                tbCompoCol3.Text = TextePourComportementCheckbox(college.CompositionTechniciens);
                tbCompoCol4.Text = TextePourComportementCheckbox(college.CompositionAgentsMaitrise);
                tbCompoCol5.Text = TextePourComportementCheckbox(college.CompositionIngenieurs);
                tbCompoCol6.Text = TextePourComportementCheckbox(college.CompositionCadres);
                tbCompoCol7.Text = TextePourComportementCheckbox(college.CompositionAutres);
                if (college.CompositionAutres) 
                {
                    tbAutresL1.Text = college.CompositionAutresL1;
                    tbAutresL2.Text = college.CompositionAutresL2;
                    tbAutresL3.Text = college.CompositionAutresL3;
                }
            }
        }

        private void RemplirSiretsAssocies(Etablissement etablissement) 
        {
            if (etablissement == null)
            {
                return;
            }
            try
            {

                List<string> numerosSiretEtablissementsRattaches = etablissement.ListNumerosSiretEtablissementsRattaches(_Election,12);

                //Parcours des textbox tbSiretAssoc comprenant l'index de l'tablissement afin de renseigner les numros de SIRET dynamiquement
                int indexChampSiret = 1;

                foreach (string numeroSiret in numerosSiretEtablissementsRattaches)
                {
                    RemplirPartieSiretAssocie(numeroSiret, indexChampSiret, 1);
                    RemplirPartieSiretAssocie(numeroSiret, indexChampSiret, 2);
                    RemplirPartieSiretAssocie(numeroSiret, indexChampSiret, 3);
                    RemplirPartieSiretAssocie(numeroSiret, indexChampSiret, 4);
                    indexChampSiret++;
                }
            }
            catch (Exception exSiret)
            {
                LoggerUniversel.Error("*** Gnration des Cerfas *** Impossible d'imprimer les numros siret rattachs", exSiret);
            }
        }

        /// <summary>
        /// Rempli les parties du siret de l'tablissement rattach pass en paramtre
        /// </summary>
        /// <param name="numeroSiret">Numro de siret</param>
        /// <param name="indexChampSiret">L'index du champ du siret</param>
        /// <param name="indexPartieSiret">L'index de la partie du siret</param>
        private void RemplirPartieSiretAssocie(string numeroSiret, int indexChampSiret, int indexPartieSiret)
        {
            ARControl champSiret = ftrCerfa.Controls["tbSiretAssoc" + indexChampSiret + "Part" + indexPartieSiret];
            ((TextBox)champSiret).Text = EclaterPourImpressionCases(StringHelper.GetPartNumeroSiret(numeroSiret, indexPartieSiret), 2);
        }

        /// <summary>
        /// Retourne un texte  placer dans une textbox, pour simuler le comportement d'une checkbox.
        /// </summary>
        /// <param name="cbChecked"></param>
        /// <returns></returns>
        private string TextePourComportementCheckbox(bool cbChecked) 
        {
            return cbChecked ? "x" : string.Empty;
        }

        /// <summary>
        /// Affecte l'image d'entte du document, selon le type d'lection considre.
        /// </summary>
        private void AffecterImageEnteteDocument() 
        {
            ElectionCSE election = (ElectionCSE)_ResultatElectionListe.Vote;
            picEnteteCerfa.Image = ImageManip.SafeLoadImageFromFile(RessourcesMngrImages.GetPathEnteteCerfa(election.TypeVote.TypeVoteInterne.CodeType.ToString(), election.Titulaires.Value, 1));
        }

        private void ArCerfaCSE_PageStart(object sender, EventArgs e)
        {
            if (!premierePage)
            {
                pageHeader.Visible = false;
            }
            premierePage = false;
        }

        private void ArCerfaCSE_PrintProgress(object sender, EventArgs e) { }

        private void detail_Format(object sender, EventArgs e) { }

        /// <summary>
        /// Remplit les zones IV (membres du bureau de vote) et IX (personne  contacter dans l'entreprise)
        /// TODO : Remplir les OS rattaches pour la zone IV
        /// NB : si une personne est  la fois prsident et assesseur, elle n'apparat qu'une fois.
        /// NB2 : la zone IV ne contient que 7 "places" pour les membres du bureau de vote ; s'il y en a plus, ils n'apparatront pas.
        /// </summary>
        /// <param name="bv">Le bureau de vote du tour 1</param>
        private void RemplirInformationsSupplementaires(BureauVote bv)
        {
            if (bv != null)
            {
                // assesseurs du bureau de vote (pour le tour 1)
                List<Personne> membres = GestionnaireCleAssesseur.GetAssesseursPourCerfa(true, bv);

                //S'il existe un prsident de bureau de vote alors on l'affiche sur le CERFA sinon on affiche le prsident de session si prsent
                if (bv.PresidentDuBureau != null)
                {
                    // si le prsident du bureau n'est pas dj prsent (i.e. s'il n'est pas dj assesseur), insertion en tte de liste
                    if (!membres.Contains(bv.PresidentDuBureau))
                        membres.Insert(0, bv.PresidentDuBureau);
                }
                else
                {
                    if (AccesMetier.SessionVote.PresidentSession != null && !membres.Contains(AccesMetier.SessionVote.PresidentSession))
                    {
                        membres.Insert(0, AccesMetier.SessionVote.PresidentSession);
                    }
                }
                // remplissage des 7 "places disponibles" pour le tour 1
                txtNomPrenomMembre1Tour1.Text = txtNomPrenomMembre1.Text = membres.Count > 0 ? membres[0].NomPrenom : string.Empty;
                txtNomPrenomMembre2Tour1.Text = txtNomPrenomMembre2.Text = membres.Count > 1 ? membres[1].NomPrenom : string.Empty;
                txtNomPrenomMembre3Tour1.Text = txtNomPrenomMembre3.Text = membres.Count > 2 ? membres[2].NomPrenom : string.Empty;
                txtNomPrenomMembre4Tour1.Text = txtNomPrenomMembre4.Text = membres.Count > 3 ? membres[3].NomPrenom : string.Empty;
                txtNomPrenomMembre5Tour1.Text = txtNomPrenomMembre5.Text = membres.Count > 4 ? membres[4].NomPrenom : string.Empty;
                txtNomPrenomMembre6Tour1.Text = membres.Count > 5 ? membres[5].NomPrenom : string.Empty;
                txtNomPrenomMembre7Tour1.Text = membres.Count > 6 ? membres[6].NomPrenom : string.Empty;
            }


            //Informations du contact de l'entreprise
            txtContactNom.Text = _ResultatElectionListe.Vote.EtablissementCollege.Etablissement.CerfaContactNom;
            txtContactPrenom.Text = _ResultatElectionListe.Vote.EtablissementCollege.Etablissement.CerfaContactPrenom;
            txtContactFonction.Text = _ResultatElectionListe.Vote.EtablissementCollege.Etablissement.CerfaContactFonction;
            txtContactNumTel.Text = _ResultatElectionListe.Vote.EtablissementCollege.Etablissement.CerfaContactTelephone;
            txtContactNumTelecopie.Text = _ResultatElectionListe.Vote.EtablissementCollege.Etablissement.CerfaContactTelecopie;
            txtContactEmail.Text = _ResultatElectionListe.Vote.EtablissementCollege.Etablissement.CerfaContactEmail;

            if (!string.IsNullOrEmpty(_ResultatElectionListe.Vote.EtablissementCollege.Etablissement.CerfaCachetEntreprise))
                rtbCachet.Text = _ResultatElectionListe.Vote.EtablissementCollege.Etablissement.CerfaCachetEntreprise;
        }

        private void ftrCerfa2_Format(object sender, EventArgs e)
        {
            picMembresBureau.Image = ImageManip.SafeLoadImageFromFile(RessourcesMngrImages.GetPathImgCerfa("Tab_Membres_Bureau_Tour1.png"));
            picCerfaTour1Ftr.Image = ImageManip.SafeLoadImageFromFile(RessourcesMngrImages.GetPathImgCerfa("Cerfa_Tour1_ftr.png"));
            //// Impression de la composition des ventuelles intersyndicales
            //if (AccesMetier.SessionVote.NumeroTour == 2)
            //{
            //    subRepListesCommunes.Report = new ArCerfaResListesCommunes(_XmlResultatsTour1);
            //}
            //else
            //{
            //    subRepListesCommunes.Report = new ArCerfaResListesCommunes(_Election.GetCriteriaListesCandidats()
            //                                                                        .Add(Expression.IsNotNull("lc.GroupementSyndicat"))
            //                                                                        .AddOrder(Order.Asc("lc.Position"))
            //                                                                        .List<ListeCandidats>(), _ResultatElectionListe);
            //}
        }

    }
}